#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
# Copyright (C) 2011, Sylvain Raybaud <sylvain.raybaud@loria.fr>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Sylvain Raybaud nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Sylvain Raybaud <sylvain.raybaud@loria.fr> ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Sylvain Raybaud BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#

from optparse import OptionParser
from mytools import open_plain_or_gz,warn,error,verbose_msg,openfiles,closefiles,getline
import sys

REPLACE = {"E":"ai",
           "O":"au",
           "J":"gn",
           "S":"ch",
           "Z":"ge",
           "R":"r",
           "H":"h"
           }

FILTER = ["hh",
          "bb",
          "xx",
         ]

FILTER_WORDS = ["<s>","</s>"]

REMOVE = [
          "sil"
         ]

def convert_bdlex(fname,outbdlex=False,do_filter=True):
    
    w_list = {}
    
    f = open_plain_or_gz(fname)
    l = f.readline()
    
    while l!="":
        
        toks = l.split()
        w = toks[0]
        if (not do_filter) or not (w in FILTER_WORDS):
            ph = toks[2:]
            
            if w in w_list:
                w_list[w] += 1
            else:
                w_list[w] = 1
            
            skip = False
            
            ph_new = []
            for p in ph:
                if do_filter and p in FILTER:
                    skip = True
                    break
                if (not do_filter) or not (p in REMOVE):
                    ph_new.append(REPLACE[p] if (do_filter and p in REPLACE) else p)
            
            if not skip:
                if outbdlex:
                    print("%s %s %s" % (w,toks[1]," ".join(ph_new)))
                else:
                    print("%s%s\t%s" % (w,(("(%i)" % w_list[w]) if w_list[w]>1 else "")," ".join(ph_new)))
        
        l = f.readline()
    
    f.close()

def convert_tt(fname,do_filter):
    
    f = open_plain_or_gz(fname)
    l = f.readline()
    
    while l!="":
        
        toks = l.strip().split("|||",1)
        s = toks[0].split()
        
        s_new = []
        for p in s:
            if (not do_filter) or not (p in REMOVE or p in FILTER):
                s_new.append(REPLACE[p] if (do_filter and p in REPLACE) else p)
        
        print("%s |||%s" % (" ".join(s_new),toks[1]))
        
        l = f.readline()
    
    f.close()

def convert_text(do_filter):
    f = sys.stdin
    l = f.readline()
    while l!="":
        s = l.split()
        s_new = []
        for p in s:
            if (not do_filter) or not (p in REMOVE or p in FILTER):
                s_new.append(REPLACE[p] if (do_filter and p in REPLACE) else p)
        
        print("%s" % " ".join(s_new))
        
        l = f.readline()

if __name__=='__main__':
    parser = OptionParser(usage="Usage: %prog [options] [bdlex|translation_table] [< input_text] > res")
    parser.add_option('-b','--bdlex',default=None,metavar="FILE",dest="bdlex",help="Convert a bdlex dictionary [%default]")
    parser.add_option('-B','--keep-bdlex',action="store_true",default=False,dest="outbdlex",help="Result is in bdlex format (instead of CMU) [%default]")
    parser.add_option('-F','--no-filter',action="store_false",default=True,dest="do_filter",help="Keep all phonemes as they are [default=filter]")
    parser.add_option('-t','--translation-table',default=None,metavar="FILE",dest="tt",help="Convert a giza translation table [%default]")
    opts,args = parser.parse_args()
    if len(args)!=0 or (opts.bdlex and opts.tt):
        parser.print_help()
        error("invalid command line.",1)
    
    if opts.bdlex:
        convert_bdlex(opts.bdlex,opts.outbdlex,opts.do_filter)
    elif opts.tt:
        convert_tt(opts.tt,opts.do_filter)
    else:
        convert_text(opts.do_filter)
