#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
# Copyright (C) 2009, Sylvain Raybaud <sylvain.raybaud@crans.org>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Sylvain Raybaud nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Sylvain Raybaud <sylvain.raybaud@crans.org> ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Sylvain Raybaud BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#

#if __name__=='__main__':

#shall return (distance,is_aligned,alignment,edition_operations_1to2)

from numpy import *

DEL = 0
INS = 1
SUB = 2
operation_codes = ['DEL','INS','SUB']

match_eq = lambda x,y: x==y

def levenshtein(s1,s2,match = match_eq):
    l1 = len(s1)+1
    l2 = len(s2)+1
    D = array([0]*l1*l2)
    D.shape = l1,l2
    Op = array([0]*l1*l2)
    Op.shape = l1,l2
    Match = array([False]*l1*l2)
    Match.shape = l1,l2
    for i in range(1,l1):
        D[i][0] = i
    for i in range(1,l2):
        D[0][i] = i
        Op[0][i] = 1
    for i in range(1,l1):
        for j in range(1,l2):
            if match(s1[i-1],s2[j-1]):
                cost = 0
                Match[i][j] = True
            else:
                cost = 1
            new_dist = [D[i-1][j]+1,D[i][j-1]+1,D[i-1][j-1]+cost] # DEL, INS, SUBS
            D[i][j] = min(new_dist)
            Op[i][j] = new_dist.index(D[i][j])
    align = []
    aligned = []
    ops1 = []
    i = l1-1
    j = l2-1
    #print "D:\n",D,"\nOp:\n",Op,"\nM:\n",Match
    while i!=0 or j!=0:
        if Op[i][j]==DEL:
            aligned.append(-1)
            ops1.append(DEL)
            align.append((s1[i-1],None))
            i -= 1
        elif Op[i][j]==INS:
            ops1.append(INS)
            align.append((None,s2[j-1]))
            j -= 1
        elif Op[i][j]==SUB:
            aligned.append(j-1 if Match[i][j] else -1)
            align.append((s1[i-1],s2[j-1]))
            ops1.append(SUB)
            i -= 1
            j -= 1
        else:
            import sys
            sys.stderr.write("levenshtein.py:\n*** ERROR *** Op matrix is inconsistent. Something is seriously broken.\n")
            exit(1)
    align.reverse()
    aligned.reverse()
    ops1.reverse()
    return D[l1-1][l2-1],aligned,align,ops1
