#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
# Copyright (C) 2009, Sylvain Raybaud <sylvain.raybaud@crans.org>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Sylvain Raybaud nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Sylvain Raybaud <sylvain.raybaud@crans.org> ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Sylvain Raybaud BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#

#if __name__=='__main__':

import sys
import gzip
import os
import random
from math import exp,log,sqrt
random.seed()

add = lambda x,y: x+y
minus = lambda x,y: x-y
mul = lambda x,y: x*y
div = lambda x,y: x/y


def dictree_ins(dic,path,val):
    """ in place value insertion in tree """
    try:
        if len(path)==0:
            raise Exception("Path must not be empty")
    except:
        raise Exception("Path must be a non empty iterable")
    if len(path)==1:
        dic[path[0]] = val
        return dic
    w = path[0]
    if not w in dic:
        dic[w] = {}
    dic[w] = dictree_ins(dic[w],path[1:],val)
    return dic

def dictree_get(dic,path,allow_prefix=False,allow_subtree=False):
    """ in place value insertion in tree """
    try:
        if len(path)==0:
            raise Exception("Path must not be empty")
    except:
        raise Exception("Path must be a non empty iterable")
    if type(dic)!=type({}):
        return dic if allow_prefix else None
    if len(path)==1:
        if path[0] not in dic:
            return None
        elif type(dic[path[0]])==type({}):
            return dic[path[0]] if allow_subtree else None
        else:
            return dic[path[0]]
    else:
        return dictree_get(dic[path[0]],path[1:],allow_prefix,allow_subtree) if path[0] in dic else None

def count_overlap(string,pattern):
    l = len(pattern)
    n = 0
    for i,c in enumerate(string):
        if c==pattern[0] and string[i:i+l]==pattern:
            n += 1
    return n

class myFile:
    def __init__(self,fname,mode="r",encoding="utf8"):
        self.filename = fname
        self.file = open_plain_or_gz(fname,mode)
        self.type = type(self.file)
        self.encoding = encoding
    
    def readline(self):
        return self.file.readline().decode() if self.type==gzip.GzipFile else self.file.readline()
    
    def write(self,l):
        return self.file.write(l.encode(self.encoding)) if self.type==gzip.GzipFile else self.file.write(l)
    
    def close(self):
        self.file.close()

def open_plain_or_gz(filename,mode='r'):
    """Transparently open gziped or plain files, based on file extension"""
    fname = str(filename)
    return gzip.open(fname,mode) if fname.endswith(".gz") else open(fname,mode)

#backward compatibility
GZ_PLAIN_open = open_plain_or_gz

def pick_random_indices(n,proportion=.5):
    N = (1-proportion)*n-0.5
    indices = range(0,n)
    while N>0:
        indices.remove(random.choice(indices))
        N -= 1
    return indices


#implemented in pyhton>=2.6
def triangular(low,high,mode):
    p =(high-mode)/(high-low)
    if random.random()<p:
        return random.uniform(low,mode)
    else:
        return random.uniform(mode,high)


def closefiles(files):
    """Close all files in an array"""
    for f in files:
        if f:
            f.close()

def openfiles(filenames,mode="r"):
    """Open all files in an array"""
    files = []
    for f in filenames:
        file = GZ_PLAIN_open(f,mode)
        files.append(file)
    return files

def getbinpath(binname):
    """Given a name look for complete path into PATH environnment variable"""
    path = os.getenv("PATH")
    paths = path.split(os.pathsep)
    for p in paths:
        binpath = p+"/"+binname
        if os.access(binpath,os.X_OK):
            return binpath
    return binname

def verbose_msg(verbose,msg):
    if verbose:
        sys.stderr.write(msg+'\n')

def debug(msg):
    """Print a warning message to stderr. The program continues."""
    sys.stderr.write("*** DEBUG *** %s\n" % msg)

def warn(msg):
    """Print a warning message to stderr. The program continues."""
    sys.stderr.write("*** WARNING *** %s\n" % msg)

def error(msg,code=1):
    """Print an error message to stderr and exists with the given code (default: 1)."""
    sys.stderr.write("*** ERROR *** %s\n" % msg)
    exit(code)

def readnlines(stream,n):
    lines = []
    i=0
    while i<n:
        lines.append(stream.readline())
        i += 1
    return lines

def getline(stream,comments=['#'],ignore=["\n"]):
    line = None
    while line!="" and (line==None or (line[0] in comments) or (line in ignore)):
        line = stream.readline()
    return line

def getnlines(stream,n,comments=['#'],ignore=["\n"]):
    lines = []
    i=0
    while i<n:
        lines.append(getline(stream,comments,ignore))
        i += 1
    return lines

def log_b(x,b):
    return log(x)/log(b)

def dbg(s):
    sys.stderr.write(str(s).strip()+"\n")

LOG2 = log(2)
def log2(x):
    return log(x)/LOG2
