#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#
# Copyright (C) 2011, Sylvain Raybaud <sylvain.raybaud@loria.fr>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Sylvain Raybaud nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Sylvain Raybaud <sylvain.raybaud@loria.fr> ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Sylvain Raybaud BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#

from optparse import OptionParser
from mytools import open_plain_or_gz,warn,error,verbose_msg,openfiles,closefiles,getline,debug
from phonetizer import Phonetizer,EPSILON
from levenshtein import levenshtein
import sys
import re

ignored_time = 0
ignored_surface = 0

def init_CM(phonemes,epsilon):
    CM = {}
    p_counts = {}
    for p_hyp in phonemes:
        CM[p_hyp] = {}
        p_counts[p_hyp] = 0
        for p_ref in phonemes:
            CM[p_hyp][p_ref] = 0
        CM[p_hyp][epsilon] = 0
    return CM,p_counts

def parse_ctm_l(l):
    toks = l.split()
    if len(toks)<5:
        return None,0,0
    t_start = float(toks[2])
    t_end = t_start+float(toks[3])
    return toks[4],t_start,t_end

buff_p = ""
buff_t_start = -1
buff_t_end = 999999

IGNORE = ["SIL",""," "]

def get_time_consistent_segments(stm,ctm,options):
    #idea: read 1 line of stm. read ctm until time goes over stm's
    
    global buff_p,buff_t_start,buff_t_end,ignored_time,ignored_surface
    
    ref_l = getline(stm,comments=[';'])
    #debug(ref_l)
    ref_toks = ref_l.split(" ",6)
    ref_s = re.sub(r"\(\%[^\)]*\)","",ref_toks[6]).strip() if len(ref_toks)>6 else ""
    while ref_l!=""  and (len(ref_s)==0 or ref_s=="ignore_time_segment_in_scoring"):
        ref_l = getline(stm,comments=[';'])
        ref_toks = ref_l.split(" ",6)
        ref_s = re.sub(r"\(\%[^\)]*\)","",ref_toks[6]).strip() if len(ref_toks)>6 else ""
    if ref_l=="":
        return "",[]
    
    t_start = float(ref_toks[3])
    t_end = float(ref_toks[4])
    
    #debug("stm: %f -> %f" % (t_start,t_end))
    
    #ref_s = ref_toks[6].split()
    #ref_alts = list(map(lambda s: s.split(","),ref_s))
    
    ctm_p = [buff_p] if buff_t_start>=t_start and buff_t_start<t_end and not (buff_p in IGNORE) else []
    
    ctm_l = getline(ctm)
    p,t_start_ctm,t_end_ctm = parse_ctm_l(ctm_l)
    #debug("ctm: %s %f->%f" % (p,t_start_ctm,t_end_ctm))
    while ctm_l!="" and t_end_ctm<=t_start:
        ignored_time += 1
        ctm_l = getline(ctm)
        p,t_start_ctm,t_end_ctm = parse_ctm_l(ctm_l)
        #debug("ctm: %s %f->%f" % (p,t_start_ctm,t_end_ctm))
    
    while ctm_l!="" and t_end_ctm>=t_start and t_start_ctm<t_end:
        if not (p in IGNORE):
            ctm_p.append(p) 
        else:
            ignored_surface += 1
        ctm_l = getline(ctm)
        p,t_start_ctm,t_end_ctm = parse_ctm_l(ctm_l)
        #debug("ctm: %s %f->%f" % (p,t_start_ctm,t_end_ctm))
    
    buff_p = p
    buff_t_start = t_start_ctm
    buff_t_end = t_end_ctm
    
    #return ref_alts,ctm_p
    return ref_s,ctm_p

#Matrix[a][b] = probability that phoneme a (in hyp) was in fact phoneme b
def confusion_matrix(dicfname,bdlexfname,stmfname,ctmfname,options):
    P = Phonetizer(dicfname,bdlexfname)
    phonemes = P.get_phonemes()
    CM,p_count = init_CM(phonemes,EPSILON)
    #debug(str(phonemes))

    stm = open_plain_or_gz(stmfname)
    ctm = open_plain_or_gz(ctmfname)

    ref,hyp = get_time_consistent_segments(stm,ctm,options)
    #while ref!=[] or hyp!=[]:
    while ref!="":
        
        #debug(str(ref))
        debug(ref)
        #debug(str(hyp))
        
        ref_ph_alts = P.phonetize_expand_alts(ref)
        ref_ph_alts_matches = list(map(lambda r:(r.split(),levenshtein(r.split(),hyp)),ref_ph_alts))
        best_match = max(ref_ph_alts_matches,key=lambda c:c[1][0])
        ref_ph_best_matches = filter(lambda c:c[1][0]==best_match,ref_ph_alts_matches)
        
        #debug(str(alignment))
        
        conf = []
        for _ in hyp:
            conf.append(set())
        conf.append(set()) #DEBUG!
        
        for (_,(_,_,a,_)) in ref_ph_best_matches:
            i = 0
            for pr,p in a:
                debug("%s - %s" % (pr,p))
                if p is not None:
                    conf[i].add(pr if pr is not None else EPSILON)
                    i += 1
        
        for i,p in enumerate(hyp):
            if p in conf[i]:
                CM[p][p] += 1
            else:
                for pr in conf[i]:
                    CM[p][pr] += 1
        
        ref,hyp = get_time_consistent_segments(stm,ctm,options)
    
    for p in phonemes:
        conf_array = sorted(list(CM[p].items()),key=lambda t:t[1],reverse=True)
        for p_ref,count in conf_array:
            print("%s %s %f" % (p,p_ref,float(count)/p_count[p]))
    
    if options.counts:
        f = open_plain_or_gz(options.counts,"w")
        for p in phonemes:
            f.write("%s %i\n" % (p,p_count[p]))
        f.close()
    
    stm.close()
    ctm.close()

if __name__=='__main__':
    parser = OptionParser(usage="Usage: %prog [options] dic bdlex stm ctm > matrix")
    parser.add_option('-c','--counts',default=None,metavar="FILE",dest="counts",help="Store phonemes counts [%default]")
    parser.add_option('-v','--verbose',default=False,action="store_true",dest="verbose",help="verbosity [%default]")
    opts,args = parser.parse_args()
    if len(args)!=4:
        parser.print_help()
        error("bad command line.",1)

    confusion_matrix(args[0],args[1],args[2],args[3],opts)
    
    verbose_msg(opts.verbose,"[INFO] items ignored in ctm:")
    verbose_msg(opts.verbose,"[INFO] time:\t%i" % ignored_time)
    verbose_msg(opts.verbose,"[INFO] surface:\t%i" % ignored_surface)
