#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#
# Copyright (C) 2011, Sylvain Raybaud <sylvain.raybaud@loria.fr>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Sylvain Raybaud nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Sylvain Raybaud <sylvain.raybaud@loria.fr> ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Sylvain Raybaud BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#

from optparse import OptionParser
from mytools import open_plain_or_gz,warn,error,verbose_msg,openfiles,closefiles,getline,debug
from phonetizer import Phonetizer
import sys

def phonetize_stm(dicfname,bdlexfname,stmfname):
    P = Phonetizer(dicfname,bdlexfname)

    stm = open_plain_or_gz(stmfname)

    l = stm.readline()
    while l!='':
        #debug(l)
        if len(l)<2 or l[0:2]==';;':
            print(l)
        else:
            prefix,s_ = l.split('>')
            s = s_.strip()
            if len(s)>0:
                p = P.phonetize_compact_alts(s)
                p_s = ""
                for alts in p:
                    p_s += " "+",".join(alts)
                print("%s>%s\n" % (prefix,p_s))
        l = stm.readline()

    stm.close()

if __name__=='__main__':
    parser = OptionParser(usage="Usage: %prog [options] dic bdlex stm > stm_ph")
    parser.add_option('-v','--verbose',default=False,action="store_true",dest="verbose",help="verbosity [%default]")
    opts,args = parser.parse_args()
    if len(args)!=3:
        parser.print_help()
        error("bad command line.",1)

    phonetize_stm(args[0],args[1],args[2])
